@@ -1,3 +1,4 @@
+import json
 import logging
 from collections import defaultdict
 
@@ -35,25 +36,49 @@ from .query import (
     BoundQuery,
     DateTimeType,
     DateType,
+    JSONType,
+    NumberArrayType,
     NumberChoiceType,
     NumberType,
+    StringArrayType,
     StringChoiceType,
     StringType,
+    UnknownType,
 )
 
+try:
+    from django.contrib.postgres.fields import ArrayField
+except ModuleNotFoundError:  # pragma: postgres
+    ArrayField = None.__class__
+
+try:
+    from django.fields import JSONField
+except ImportError:  # pragma: json field
+    try:
+        from django.contrib.postgres.fields import JSONField
+    except ModuleNotFoundError:  # pragma: postgres
+        JSONField = None.__class__
+
+
+_STRING_FIELDS = (
+    models.CharField,
+    models.TextField,
+    models.GenericIPAddressField,
+    models.UUIDField,
+)
+_NUMBER_FIELDS = (
+    models.DecimalField,
+    models.FloatField,
+    models.IntegerField,
+    models.AutoField,
+)
 _FIELD_TYPE_MAP = {
     models.BooleanField: BooleanType,
     models.NullBooleanField: BooleanType,
-    models.CharField: StringType,
-    models.TextField: StringType,
-    models.GenericIPAddressField: StringType,
-    models.UUIDField: StringType,
     models.DateTimeField: DateTimeType,
     models.DateField: DateType,
-    models.DecimalField: NumberType,
-    models.FloatField: NumberType,
-    models.IntegerField: NumberType,
-    models.AutoField: NumberType,
+    **{f: StringType for f in _STRING_FIELDS},
+    **{f: NumberType for f in _NUMBER_FIELDS},
 }
 
 
@@ -141,13 +166,29 @@ def _get_calculated_field(request, field_name, model_name, model, admin, model_f
                 choices=choices,
             )
     else:
-        return OrmCalculatedField(
-            model_name=model_name, name=field_name, pretty_name=field_name, admin=admin
-        )
+        if not getattr(getattr(admin, field_name, None), "ddb_hide", False):
+            return OrmCalculatedField(
+                model_name=model_name,
+                name=field_name,
+                pretty_name=field_name,
+                admin=admin,
+            )
+        else:
+            return None
 
 
 def _get_field_type(model, field_name, field):
-    if field.__class__ in _FIELD_TYPE_MAP:
+    if isinstance(field, ArrayField) and isinstance(
+        field.base_field, _STRING_FIELDS
+    ):  # pragma: postgres
+        return StringArrayType, field.base_field.choices
+    elif isinstance(field, ArrayField) and isinstance(
+        field.base_field, _NUMBER_FIELDS
+    ):  # pragma: postgres
+        return NumberArrayType, field.base_field.choices
+    elif isinstance(field, JSONField):  # pragma: json field
+        return JSONType, None
+    elif field.__class__ in _FIELD_TYPE_MAP:
         res = _FIELD_TYPE_MAP[field.__class__]
     else:
         for django_type, field_type in _FIELD_TYPE_MAP.items():
@@ -159,7 +200,9 @@ def _get_field_type(model, field_name, field):
                 logging.getLogger(__name__).warning(
                     f"DDB {model.__name__}.{field_name} unsupported type {type(field).__name__}"
                 )
-            return None, None
+            return UnknownType, None
+
+    # Choice fields have different lookups
     if res is StringType and field.choices:
         return StringChoiceType, field.choices
     elif res is NumberType and field.choices:
@@ -196,20 +239,20 @@ def _get_fields_for_model(request, model, admin, admin_fields):
                 url_func=field.storage.url,
             )
         elif isinstance(field, type(None)):
-            fields[field_name] = _get_calculated_field(
+            orm_field = _get_calculated_field(
                 request, field_name, model_name, model, admin, model_fields
             )
+            if orm_field:
+                fields[field_name] = orm_field
         else:
             field_type, choices = _get_field_type(model, field_name, field)
-
-            if field_type:
-                fields[field_name] = OrmConcreteField(
-                    model_name=model_name,
-                    name=field_name,
-                    pretty_name=field_name,
-                    type_=field_type,
-                    choices=choices,
-                )
+            fields[field_name] = OrmConcreteField(
+                model_name=model_name,
+                name=field_name,
+                pretty_name=field_name,
+                type_=field_type,
+                choices=choices,
+            )
 
     return OrmModel(fields=fields, admin=admin)
 
@@ -238,23 +281,37 @@ def get_models(request):
     return {**models, **types}
 
 
-def _get_django_lookup(field_type, lookup):
-    if field_type == StringType and lookup == "equals":
-        return "iexact"
+def _get_django_lookup(field_type, lookup, filter_value):
+    if lookup == "field_equals":  # pragma: json field
+        lookup, filter_value = filter_value
+        return lookup, filter_value
+    elif field_type in [StringType, StringChoiceType]:
+        return (
+            {
+                "equals": "iexact",
+                "regex": "iregex",
+                "contains": "icontains",
+                "starts_with": "istartswith",
+                "ends_with": "iendswith",
+                "is_null": "isnull",
+            }[lookup],
+            filter_value,
+        )
     else:
-        lookup = {
-            "equals": "exact",
-            "regex": "iregex",
-            "contains": "icontains",
-            "starts_with": "istartswith",
-            "ends_with": "iendswith",
-            "is_null": "isnull",
-            "gt": "gt",
-            "gte": "gte",
-            "lt": "lt",
-            "lte": "lte",
-        }[lookup]
-        return lookup
+        return (
+            {
+                "equals": "exact",
+                "is_null": "isnull",
+                "gt": "gt",
+                "gte": "gte",
+                "lt": "lt",
+                "lte": "lte",
+                "contains": "contains",
+                "length": "len",
+                "has_key": "has_key",
+            }[lookup],
+            filter_value,
+        )
 
 
 def _filter(qs, filter_, filter_str):
@@ -265,12 +322,17 @@ def _filter(qs, filter_, filter_str):
         negation = True
         lookup = lookup[4:]
 
-    lookup = _get_django_lookup(filter_.orm_bound_field.type_, lookup)
+    filter_value = filter_.parsed
+    lookup, filter_value = _get_django_lookup(
+        filter_.orm_bound_field.type_, lookup, filter_value
+    )
     filter_str = f"{filter_str}__{lookup}"
+    if lookup == "contains":  # pragma: postgres
+        filter_value = [filter_value]
     if negation:
-        return qs.exclude(**{filter_str: filter_.parsed})
+        return qs.exclude(**{filter_str: filter_value})
     else:
-        return qs.filter(**{filter_str: filter_.parsed})
+        return qs.filter(**{filter_str: filter_value})
 
 
 def _cols_sub_query(bound_query):
@@ -405,9 +467,17 @@ def get_results(request, bound_query, orm_models):
         return results
 
     def get_fields(row, fields):
-        return tuple(
-            (field.queryset_path, row[field.queryset_path]) for field in fields
-        )
+        res = []
+        for field in fields:
+            v = row[field.queryset_path]
+            if isinstance(v, list):  # pragma: postgres
+                v = tuple(v)
+            try:
+                hash(v)
+            except TypeError:
+                v = json.dumps(v)
+            res.append((field.queryset_path, v))
+        return tuple(res)
 
     col_keys = {}
     for row in cols_res:
