@@ -1,3 +1,4 @@
+import json
 import urllib
 from dataclasses import dataclass
 from functools import lru_cache
@@ -191,10 +192,10 @@ class ChoiceTypeMixin:
     @staticmethod
     def format(value, choices=None):
         assert choices
-        return dict(choices)[value]
+        return dict(choices)[value] if value is not None else None
 
 
-class StringChoiceType(ChoiceTypeMixin, StringType):
+class StringChoiceType(ChoiceTypeMixin, BaseType):
     lookups = {
         **StringType.lookups,
         "equals": "stringchoice",
@@ -202,6 +203,26 @@ class StringChoiceType(ChoiceTypeMixin, StringType):
     }
 
 
+class ArrayTypeMixin:
+    default_value = None
+
+    @staticmethod
+    def format(value, choices=None):  # pragma: postgres
+        if choices:
+            value = [dict(choices)[v] if v is not None else None for v in value]
+        return ", ".join(str(v) for v in value)
+
+
+class StringArrayType(ArrayTypeMixin, BaseType):
+    lookups = {
+        "contains": "stringchoice",
+        "length": "number",
+        "not_contains": "stringchoice",
+        "not_length": "number",
+        "is_null": "boolean",
+    }
+
+
 class RegexType(BaseType):
     default_value = ".*"
 
@@ -241,7 +262,7 @@ class NumberType(BaseType):
         return float(value)
 
 
-class NumberChoiceType(ChoiceTypeMixin, NumberType):
+class NumberChoiceType(ChoiceTypeMixin, BaseType):
     lookups = {
         **NumberType.lookups,
         "equals": "numberchoice",
@@ -249,6 +270,16 @@ class NumberChoiceType(ChoiceTypeMixin, NumberType):
     }
 
 
+class NumberArrayType(ArrayTypeMixin, BaseType):
+    lookups = {
+        "contains": "numberchoice",
+        "length": "number",
+        "not_contains": "numberchoice",
+        "not_length": "number",
+        "is_null": "boolean",
+    }
+
+
 class YearType(NumberType):
     default_value = timezone.now().year
     lookups = {
@@ -395,6 +426,45 @@ class BooleanType(BaseType):
             raise ValueError("Expected 'true' or 'false'")
 
 
+class UnknownType(BaseType):
+    @staticmethod
+    def format(value, choices=None):
+        assert not choices
+        return str(value)
+
+    lookups = {"is_null": "boolean"}
+
+
+class JSONFieldType(BaseType):
+    default_value = "|"
+
+    @staticmethod
+    def _parse(value):  # pragma: json field
+        value = value.strip()
+        if "|" not in value:
+            raise ValueError("Missing seperator '|'")
+        field, value = value.split("|", 1)
+        if not field:
+            raise ValueError("Invalid field name")
+        if value.startswith("{") or value.startswith("["):
+            raise ValueError("Not a JSON primitive")
+        try:
+            value = json.loads(value)
+        except json.JSONDecodeError:
+            raise ValueError("Not a JSON primitive")
+        return [field, value]
+
+
+class JSONType(BaseType):
+    lookups = {
+        "is_null": "boolean",
+        "has_key": "string",
+        "field_equals": "jsonfield",
+        "not_has_key": "string",
+        "not_field_equals": "jsonfield",
+    }
+
+
 def all_subclasses(cls):
     res = set()
     queue = {cls}
